module LotusAdmin
  module Authorization
    extend ActiveSupport::Concern

    include Pundit

    class_methods do
      def policy_class(policy_class_or_name)
        self._policy_class_name = policy_class_or_name.to_s
      end
    end

    included do
      class_attribute :_policy_class_name, instance_accessor: false

      rescue_from Pundit::NotAuthorizedError, with: :user_not_authorized
    end

    private

    def pundit_user
      current_administrator
    end

    def beginning_of_association_chain
      chain = super

      if using_pundit?
        policy_scope(chain, policy_scope_class: "#{ self.class._policy_class_name }::Scope".constantize)
      else
        chain
      end
    end

    def user_not_authorized
      redirect_to root_path, alert: "You are not authorized to perform this action."
    end

    def using_pundit?
      self.class._policy_class_name.present?
    end
  end
end
