module LotusAdmin
  module FilterableController
    extend ActiveSupport::Concern

    included do
      helper FilterHelpers

      class_attribute :filters_enabled
      self.filters_enabled = false

      class_attribute :_beginning_of_scope_chain, instance_accessor: false

      let(:filters) { self.class.filters_config }
      let(:apply_filters?) do
        params.fetch(:q, {}).to_unsafe_h.any? { |_,v| !v.blank? }
      end

      let(:filters_enabled?) { self.class.filters_enabled }
      let(:ransack_object) { beginning_of_association_chain.ransack(params[:q]) }
    end

    class_methods do
      def filter(name, options={})
        self.filters_enabled = true

        filters_config << Filters::Configuration.new(new.resource_class, name, options)
      end

      def filters_config
        @filters_config ||= []
      end

      def beginning_of_scope_chain(&block)
        self._beginning_of_scope_chain = block
      end
    end

    private

    def beginning_of_association_chain
      instance_exec(&self.class._beginning_of_scope_chain)
    end

    def end_of_association_chain
      paginate(ransack_object.result)
    end
  end
end
