class LotusAdmin::ResourceController < LotusAdmin::AuthenticatedController
  beginning_of_scope_chain { resource_class.all }

  let(:resource) { beginning_of_association_chain.find(params[:id]) }

  before_action :set_view_paths, only: [:new, :edit, :index, :show]

  def self.manages(model_class)
    self._resource_class = model_class
  end

  class_attribute :_resource_class, instance_accessor: false

  let(:menu_identifier) { "#{ resource_class.model_name.route_key }_index" }
  let(:resource_class) { self.class._resource_class }
  let(:collection_path) { url_for(resource_class) }

  def new(&block)
    self.resource = build_resource

    authorize(resource) if using_pundit?

    block.call if block.present?
  end

  def create(&block)
    self.resource = build_resource(permitted_params)

    authorize(resource) if using_pundit?

    respond_to do |format|
      format.html

      if block.present?
        block.call(format)
      else
        if resource.save
          redirect_to after_create_redirect, notice: "Created new #{ resource_class.model_name.human } successfully"
        else
          render :new
        end
      end
    end
  end

  def edit(&block)
    authorize(resource) if using_pundit?

    if block.present?
      block.call
    else
      render :edit
    end
  end

  def update(&block)
    authorize(resource) if using_pundit?

    respond_to do |format|
      format.html

      if block.present?
        block.call(format)
      else
        if resource.update(permitted_params)
          redirect_to after_update_redirect, notice: "Updated #{ resource_class.model_name.human } successfully"
        else
          render :edit
        end
      end
    end
  end

  def index(&block)
    authorize(resource_class) if using_pundit?

    respond_to do |format|
      format.html

      block.call(format) if block.present?
    end
  end

  def show(&block)
    authorize(resource) if using_pundit?

    respond_to do |format|
      format.html

      block.call(format) if block.present?
    end
  end

  def destroy(&block)
    authorize(resource) if using_pundit?

    respond_to do |format|
      format.html

      if block.present?
        block.call(format)
      else
        if resource.destroy
          flash[:notice] = "#{ resource_class.model_name.human } has been removed"
        else
          flash[:error] = "There was an error removing that #{ resource_class.model_name.human }"
        end

        redirect_to after_destroy_redirect
      end
    end
  end

  private

  def build_resource(params = {})
    resource_class.new(params)
  end

  def association_name
    resource_class.model_name.plural
  end

  def set_view_paths
    append_view_path("lotus_admin/resource/#{ action_name }")
  end

  def after_update_redirect
    [lotus_admin, resource]
  end

  def after_create_redirect
    after_update_redirect
  end

  def after_destroy_redirect
    [lotus_admin, resource_class]
  end
end
