module LotusAdmin
  module LinkHelpers
    def default_table_actions_column(resource)
      render 'lotus_admin/shared/table/actions', resource: resource
    end

    def undecorated(resource)
      resource.try(:object).presence || resource
    end

    def view_resource_item(resource)
      if policy(resource).show?
        content_tag(:li) { view_link(resource) }
      end
    end

    def new_resource_item(parent = nil)
      if policy(resource_class).new?
        content_tag(:li) { new_link(parent) }
      end
    end

    def edit_resource_item(resource)
      if policy(resource).edit?
        content_tag(:li) { edit_link(resource) }
      end
    end

    def destroy_resource_item(resource)
      if policy(resource).destroy?
        content_tag(:li) { destroy_link(resource) }
      end
    end

    def view_link(resource)
      link_to lotus_admin.polymorphic_path(undecorated(resource)), title: 'View Details' do
        content_tag(:i, nil, class: 'zmdi zmdi-eye')
      end
    end

    def new_link(parent = nil)
      link_to lotus_admin.new_polymorphic_path([parent, resource_class].compact), title: 'New' do
        content_tag(:i, nil, class: 'zmdi zmdi-plus-circle')
      end
    end

    def edit_link(resource)
      link_to lotus_admin.edit_polymorphic_path(undecorated(resource)), title: 'Edit' do
        content_tag(:i, nil, class: 'zmdi zmdi-edit')
      end
    end

    def destroy_link(resource)
      link_to lotus_admin.polymorphic_path(undecorated(resource)), title: 'Delete', data: { method: :delete, confirm: "Are you sure you want to remove this #{ resource.model_name.human }?" } do
        content_tag(:i, nil, class: 'zmdi zmdi-delete')
      end
    end
  end
end
