module LotusAdmin
  class Exporter
    class_attribute :_columns, instance_accessor: false
    class_attribute :_decorates, instance_accessor: false
    class_attribute :_resource_class, instance_accessor: false

    self._columns = []
    self._decorates = false

    # reset so that each subclass has it's own collection
    def self.inherited(other)
      other._columns = []
    end

    class Column
      attr_reader :attribute_name, :title, :block
      def initialize(attribute_name, title, &block)
        @attribute_name = attribute_name
        @title = title
        @block = block || ->(model){ model.public_send(attribute_name) }
      end

      def data(model)
        block.call(model)
      end
    end

    class << self
      def stream(*args)
        new.stream(*args)
      end

      def generate(*args)
        new.generate(*args)
      end

      private

      def column(attribute_name, title = nil, &block)
        self._columns << Column.new(attribute_name, title, &block)
      end

      def decorates(boolean)
        self._decorates = boolean
      end

      def model(resource_class)
        self._resource_class = resource_class
      end
    end

    def generate(collection)
      @collection = collection

      CSV.generate do |csv|
        csv << headers

        content.each do |row|
          csv << row
        end
      end
    end

    def stream(collection, stream)
      stream.write CSV.generate_line(headers)

      @collection = collection

      content.each do |row|
        stream.write CSV.generate_line(row)
      end
    end

    private

    def view_context
      @view_context ||= begin
        ActionView::Base.new(ActionController::Base.view_paths).tap do |view|
          view.class_eval do
            include Rails.application.routes.url_helpers
            include ::ApplicationHelper if defined?(::ApplicationHelper)
          end
        end
      end
    end

    def headers
      self.class._columns.map do |column|
        column.title.presence || self.class._resource_class.human_attribute_name(column.attribute_name)
      end
    end

    def content
      each_item.map do |record|
        record = record.decorate(view_context) if decorator?

        self.class._columns.map do |column|
          column.data(record)
        end
      end
    end

    def each_item(&block)
      return @collection.each.lazy unless @collection.respond_to?(:find_each)

      @collection.find_each(batch_size: 500).lazy
    end

    def decorator?
      self.class._decorates == true
    end
  end
end
