module LotusAdmin
  class Router
    def install(router, at: 'admin')
      router.devise_for :administrators, devise_config

      router.mount LotusAdmin::Engine, at: at
    end

    def resources(router)
      resource_routes.each do |routes|
        router.instance_eval(&routes)
      end
    end

    def define(&block)
      resource_routes.push(block)
    end

    def resource_routes
      @resource_routes ||= []
    end

    private

    def devise_config
      {
        class_name: user_class_name,
        skip: :registrations,
        controllers: devise_controllers
      }
    end

    def devise_controllers
      {
        confirmations: 'lotus_admin/confirmations',
        sessions: 'lotus_admin/sessions',
        passwords: 'lotus_admin/passwords'
      }
    end

    def user_class_name
      LotusAdmin.configuration.user_class_name
    end
  end
end
